(* Syntaxe abstraite des expression de type *)
type type_expr = Integer | Boolean | Array of type_expr;;
type var_list = (string * type_expr) list;;

type program = {
    global_vars : var_list; 
    (* variables globales *)
    definitions : (string * definition) list;
    (* fonctions et procdures globales *)
    main : instruction list;
    (* corps principal du programme *)
  } 
and definition = {
    arguments : var_list; result : type_expr option;
    (* arguments  et type du rsultat *)
    local_vars : var_list;
    (* variables locales *)
    body : instruction list;
    (* corps de la fonction *)
  }

and expression =
  | Int of int | Bool of bool
    (* constantes *)
  | Bin of binop * expression * expression
    (* expressions arithtiques *)
  | Get of string
    (* accs  une variable *)
  | Function_call of string * expression list
    (* appel de fonction *)
  | Geti of expression * expression
    (* accs dans un tableau  une position *)
  | Alloc of expression * type_expr
    (* Cration d'un tableau d'une certaine taille *)

and binop =
  | Plus | Minus | Times | Div
    (* arithmtique *)
  | Lt | Le | Gt | Ge | Eq | Ne 
    (* comparaisons *) 

and instruction = 
  | Set of string * expression
    (* Affectation d'une variable *)
  | Sequence of instruction list
    (* Suite d'instructions *)
  | If of expression * instruction * instruction
  | While of expression * instruction
  | Procedure_call of string * expression list
    (* Appel de procdure *)
  | Write_int of expression
  | Writeln_int of expression
    (* Ecriture d'un entier sans ou avec retour  la ligne *)
  | Read_int of string
    (* Affectation dans un tableau *)
  | Seti of expression * expression * expression
